"""Task manager configuration constants."""

DEFAULT__APPLICATION__MAX_PARALLEL_TASKS = 4
DEFAULT__APPLICATION__TASK_FETCH_INTERVAL = (
    5  # 5 seconds between task fetches when idle
)
DEFAULT__APPLICATION__LEASE_REFRESH_INTERVAL = 120  # 2 minutes in seconds
DEFAULT__APPLICATION__DEBUG_MODE = False
DEFAULT__APPLICATION__SNOWFLAKE_DATABASE_FOR_METADATA = "SNOWCONVERT_AI"
DEFAULT__APPLICATION__SNOWFLAKE_SCHEMA_FOR_DATA_MIGRATION_METADATA = "DATA_MIGRATION"
DEFAULT__SERVER__HOST = "0.0.0.0"
DEFAULT__SERVER__PORT = 5001
